#from shaonutil.stats import ClassA
from os.path import dirname, basename, isfile, join
import glob
import shaonutil

modules = glob.glob(join(dirname(__file__), "*.py"))
__all__ = [ basename(f)[:-3] for f in modules if isfile(f) and not f.endswith('__init__.py')]
for module_name in __all__:
	__import__('shaonutil'+'.'+module_name)
	getattr(shaonutil, module_name)


from os.path import dirname, basename, isfile, join
import os
import sys
import glob
import argparse		
import shaonutil
import ast

def show_info(functionNode,ModuleName,classobj=""):
    str_ = ""

    if classobj != "":
    	Parameters = [arg.arg for arg in functionNode.args.args if arg.arg != 'self']
    	classobj = getattr(__import__(ModuleName), classobj.name)
    	funcobj = getattr(classobj,functionNode.name)
    	if funcobj.__doc__ != None:
    		str_ += "    Function "+ functionNode.name +"("+ ','.join(Parameters) +") -> Description: " + funcobj.__doc__ + "\n"
    	else:
    		str_ += "    Function "+ functionNode.name+"("+ ','.join(Parameters) +")" + "\n"
    else:
    	Parameters = [arg.arg for arg in functionNode.args.args]
    	funcobj = getattr(__import__(ModuleName), functionNode.name)
    	if funcobj.__doc__ != None:
    		str_ += "Function " + functionNode.name +"("+ ','.join(Parameters) +") -> Description: " + funcobj.__doc__ + "\n"
    	else:
    		str_ += "Function " + functionNode.name +"("+ ','.join(Parameters) +")" + "\n"
    #import pdb; pdb.set_trace()
    return str_
        

def generateModuleFunctionUsageString(ModuleName):
    with open(ModuleName+".py") as file:
        node = ast.parse(file.read())

    functions = [n for n in node.body if isinstance(n, ast.FunctionDef)]
    classes = [n for n in node.body if isinstance(n, ast.ClassDef)]

    full_string = ""

    
    for function in functions:
    	full_string += show_info(function,ModuleName)

    for class_ in classes:
        classobj = getattr(__import__(ModuleName), class_.name)
        if classobj.__doc__ != None:
        	full_string += "Class " + class_.name + " -> Description: " + classobj.__doc__ + "\n"
        else:
        	full_string += "Class " + class_.name + "\n"

        methods = [n for n in class_.body if isinstance(n, ast.FunctionDef)]
        for method in methods:
            full_string += show_info(method,ModuleName,classobj=class_)

    return full_string

def generateDirFunctionUsageString():
    full_string = ""
    
    modules = glob.glob(join(dirname(__file__), "*.py"))
    __all__ = [ basename(f)[:-3] for f in modules if isfile(f) and not f.endswith('__init__.py')]
    print(__all__)
    for ModuleName in __all__:
    	print(ModuleName)
    	if ModuleName != "__init__" and ModuleName+'.py' != basename(__file__):
            #ModuleName = "mysqlDB.py"
            ModuleName = ModuleName.split('.')[0]
            Module = __import__(ModuleName)
            full_string += "### " + str(Module.__doc__) + "\n" + generateModuleFunctionUsageString(ModuleName)
    
    return full_string



def get_members(module):
	return [member for member in dir(module) if callable(getattr(module, member))]

def get_file_description_file(module):
	member_doc_dic = {}
	for member in get_members(module):
		if getattr(module, member).__doc__ != None:
			member_doc_dic[member] = getattr(module, member).__doc__
	return member_doc_dic

def createNewReadme():
	name = input("Project Name:")
	version = input("Project Version:")
	tag = input("Project Tag:")
	author = input("Project Author:")
	contact = input("Project Contact:")
	installation = input("Installation Link/String:")
	
	final_string_to_save = f"""# {name}  - {version}
## {tag}

Stable Version - {version}
Author: {author}
Contact: {contact}

## Utilities

## Installation
{installation}
## Function Usages

Function Usages End


## Versioning 

*major.minor[.maintenance[.build]]* (example: *1.4.3.5249*)

adoption: major.minor.patch.maintenance.status.trials_for_success

The last position

- 0 for alpha (status) 
- 1 for beta (status)
- 2 for release candidate
- 3 for (final) release

For instance:

- 1.2.0.1 instead of 1.2-a1
- 1.2.1.2 instead of 1.2-b2 (beta with some bug fixes)
- 1.2.2.3 instead of 1.2-rc3 (release candidate)
- 1.2.3.0 instead of 1.2-r (commercial distribution)
- 1.2.3.5 instead of 1.2-r5 (commercial distribution with many bug fixes)"""
	return final_string_to_save

def generateFunctionUsagesString(realcurrentpath):
	#list all py modules in current directory
	all_py_files = glob.glob(join(realcurrentpath, "*.py"))
	allmodules = [basename(c)[:-3] for c in all_py_files if isfile(c) and not c.endswith('__init__.py')]

	#get function usages lines
	func_usages_string = ''
	for m in allmodules:
		module_heading = '### '+m+'\n\n'
		module = __import__(m)
		member_doc_dic = get_file_description_file(module)
		func_lines = ''
		for member in member_doc_dic:
			func_lines += member + ' - ' + member_doc_dic[member] + '\n\n'
		func_usages_string += module_heading + func_lines

	return func_usages_string

def init(args):
	printline=True

	realcurrentpath = os.path.realpath('')
	sys.path.append(realcurrentpath)

	if args.output:
		outputfile = args.output
	else:
		outputfile = 'docu.md'

	outputfile = os.path.join(realcurrentpath, outputfile)

	if args.readme == 'bare':
		alllines = createNewReadme()
	else:
		readmefilename = args.readme
		filerealpath = os.path.join(realcurrentpath, readmefilename)
		# read the contents of your README file
		lines = shaonutil.file.read_file(filerealpath)
		alllines = '\n'.join(lines)

	start = '## Function Usages'
	end = 'Function Usages End'

	func_usages_string = start + '\n\n' + generateDirFunctionUsageString() + '\n\n' + end

	
	deductlines = alllines[ alllines.index(start):alllines.index(end)+len(end) ]

	#replacing existing lines with prepared lines
	final_string_to_save = alllines.replace(deductlines,func_usages_string)

	if(printline):print(final_string_to_save)
	shaonutil.file.write_file(outputfile, final_string_to_save,mode="w")

def main():
	parser = argparse.ArgumentParser(description="Automatic Documentation Generator")
	parser.add_argument("--readme", help="input the sample readme file",type=str,required=True)
	parser.add_argument("--output", help="input the outputfile file",type=str)
	args = parser.parse_args()

	try:
		init(args)
	except KeyboardInterrupt:
		sys.exit(0)

if __name__ == '__main__':
	main()



"""BarCode"""
#https://note.nkmk.me/en/python-pillow-qrcode/
#from barcode import EAN8,Code128,Code39
# encode,decode,display,actual_data,make_barcode_matrix
try:
    reduce
except NameError:
    from functools import reduce
#from io import BytesIO
from barcode.writer import ImageWriter
from imutils.video import VideoStream
from barcode import __BARCODE_MAP
from imutils.video import FPS
from pyzbar import pyzbar
from PIL import Image
import numpy as np
import shaonutil
import barcode
import imutils
import qrcode
import time
import sys
import cv2
import PIL
import os

def calculate_checksum(data):
    """Calculates the checksum for EAN13-Code / EAN8-Code return type: Integer"""
    def sum_(x, y):
        return int(x) + int(y)

    evensum = reduce(sum_, data[-2::-2])
    oddsum = reduce(sum_, data[-1::-2])
    return (10 - ((evensum + oddsum * 3) % 10)) % 10

def verify_data(data):
	"""Verify the EAN encoded data"""
	verification_digit = int(data[-1])
	check_digit = data[:-1]
	return verification_digit == calculate_checksum(check_digit)

def actual_data(decodedObjects):
	"""Returns data without checksum digit for EAN type"""
	if len(decodedObjects) > 0:
		obj = decodedObjects[0]
		data = obj.data.decode('ascii')
	else:
		return False

	if 'ean' in obj.type.lower():
		if verify_data(data):
			return data[:-1]
		else:
			return False
	else:
		return data


def encode(type_,file_,data,rt='FILE'):
	"""Encode the data as barcode or qrcode"""
	# rt = 'OBJ'
	__BARCODE_MAP['qrcode'] = ''
	if not type_.lower() in __BARCODE_MAP:
		raise ValueError("BarCode Type invalid")
	"""
	if type_ == 'EAN8':
		
		# print to a file-like object:
		#rv = BytesIO()
		#EAN8(str(1708929), writer=ImageWriter()).write(rv)
		
		# or sure, to an actual file:
		with open(file_, 'wb') as f:
		    EAN8(data, writer=ImageWriter()).write(f)
	elif type_ == 'Code128':
		with open(file_, 'wb') as f:
		    Code128(data, writer=ImageWriter()).write(f)
	elif type_ == 'Code39':
		with open(file_, 'wb') as f:
		    Code39(data, writer=ImageWriter()).write(f)
	elif type_ == 'qrcode':

		qr = qrcode.QRCode(
		    version=1,
		    error_correction=qrcode.constants.ERROR_CORRECT_L,
		    box_size=10,
		    border=4,
		)
		qr.add_data(data)
		qr.make(fit=True)

		img = qr.make_image(fill_color="black", back_color="white")
		img.save(file_)
	"""

	if type_ == 'qrcode':

		qr = qrcode.QRCode(
		    version=1,
		    error_correction=qrcode.constants.ERROR_CORRECT_L,
		    box_size=10,
		    border=4,
		)
		qr.add_data(data)
		qr.make(fit=True)

		img = qr.make_image(fill_color="black", back_color="white")
		if  rt == 'OBJ':
			return img
		else:
			img.save(file_)
			return file_
	else:
		#ean = barcode.get('ean13', '123456789102', writer = barcode.writer.ImageWriter())
		#print(type(ean))
		
		bar_class = barcode.get_barcode_class(type_)
		#bar_class.default_writer_options['write_text'] = False
		bar_class.default_writer_options['text_distance'] = .5
		bar_class.default_writer_options['quiet_zone'] = 1.8
		bar_class.default_writer_options['module_height'] = int(15/1.1)
		
		writer=ImageWriter()
		bar = bar_class(data, writer)
		#print(type(bar))
		to_be_resized = bar.render()
		#bar.save('temp')
		#to_be_resized = Image.open('temp.png')
		del bar

		width,height = to_be_resized.size
		width = int(width/1.2)
		newSize = (width, height)
		resized = to_be_resized.resize(newSize, resample=PIL.Image.NEAREST)
		
		
		if  rt == 'OBJ':
			if os.path.exists(file_): os.remove(file_)
			return resized
		else:
			resized.save(file_)
			return file_

		



	
def decode(infile,log=False):
	"""Decode barcode or qrcode"""
	if type(infile) == str:
		im = cv2.imread(infile)
	elif type(infile) == np.ndarray:
		im = infile

	data = False
	# Find barcodes and QR codes
	decodedObjects = pyzbar.decode(im)

	# Print results
	for obj in decodedObjects:
		if log: print('Type : ', obj.type)
		if log: print('Data : ', obj.data,'\n')
    
	return decodedObjects


def displayBarcode(im, decodedObjects):
	"""Mark and show the detected barcode"""
    # Loop over all decoded objects
	for decodedObject in decodedObjects: 
		points = decodedObject.polygon

        # If the points do not form a quad, find convex hull
		if len(points) > 4 : 
			hull = cv2.convexHull(np.array([point for point in points], dtype=np.float32))
			hull = list(map(tuple, np.squeeze(hull)))
		else : 
			hull = points;

        # Number of points in the convex hull
		n = len(hull)

        # Draw the convext hull
		for j in range(0,n):
			cv2.line(im, hull[j], hull[ (j+1) % n], (255,0,0), 3)

	return im

def make_barcode_matrix(type_,unique_ids,row_number,column_number,filename):
	"""Make barcode matrix image"""
	if not len(unique_ids) == row_number * column_number:
		raise ValueError("number of ids not equal to row x column size")
    
	TwoDArray = np.array(unique_ids).reshape(row_number,column_number)
	column_img = []
	for row_ids in TwoDArray:
		row_img  = [encode(type_,'',row_ids[i],rt='OBJ') for i in range(column_number)] 
		row_concatenated_img = shaonutil.image.merge_horizontally(row_img)
		column_img.append(row_concatenated_img)

	shaonutil.image.merge_vertically(column_img,filename)


def read_live_barcode(detection_threshold = 50):
	"""Live read the barcode and returns data"""
	##detection_threshold = 50 # 1 sec
	##detection_threshold = 100 # 2 sec

	detection_time = 'no detection'

	# initialize the video stream and allow the camera sensor to warm up
	print("[INFO] starting video stream...")
	vs = VideoStream(src=0).start()
	# vs = VideoStream(usePiCamera=True).start()
	time.sleep(2.0)

	# start the FPS counter
	fps = FPS().start()

	counter_i = 0
	# loop over frames from the video file stream
	previous_data = None
	while True:
		# grab the frame from the threaded video stream and resize it
		# to 500px (to speedup processing)
		frame = vs.read()
		frame = imutils.resize(frame, width=500)
		
		# convert the input frame from (1) BGR to grayscale (for face
		# detection) and (2) from BGR to RGB (for face recognition)
		gray = cv2.cvtColor(frame, cv2.COLOR_BGR2GRAY)
		rgb = cv2.cvtColor(frame, cv2.COLOR_BGR2RGB)

		im = gray
		
		decodedObjects = decode(im,log=True)
		frame = display(frame, decodedObjects)
		data = actual_data(decodedObjects)

		# draw the predicted face name on the image
		#cv2.rectangle(frame, (left, top), (right, bottom),	(0, 255, 0), 2)
		#y = top - 15 if top - 15 > 15 else top + 15
		#cv2.putText(frame, name, (left, y), cv2.FONT_HERSHEY_SIMPLEX, 0.75, (0, 255, 0), 2)

		# display the image to our screen
		cv2.imshow("Frame", frame)
		key = cv2.waitKey(1) & 0xFF

		# if the `q` key was pressed, break from the loop
		if key == 27:
			break

		# update the FPS counter
		fps.update()

		#barcode detection program
		if data:
			if previous_data == data:
				if counter_i == 1: start_time = time.time()
				counter_i += 1
			else:
				counter_i = 0
		previous_data = data

		if counter_i > detection_threshold:
			detection_time = time.time() - start_time
			break
		

	# stop the timer and display FPS information
	fps.stop()
	print("[INFO] elasped time: {:.2f}".format(fps.elapsed()))
	print("[INFO] approx. FPS: {:.2f}".format(fps.fps()))

	# do a bit of cleanup
	cv2.destroyAllWindows()
	vs.stop()
	
	message = data
	return message, detection_time


"""Mysql Database"""
from tkinter import ttk,Tk,Label,Entry
from shaonutil.strings import generateCryptographicallySecureRandomString
import tkinter as tk
import mysql.connector as mysql


class MySQL:
	"""A class for all mysql actions"""
	def __init__(self,config,log=False):
		self.log = log
		self.config = config

	def reopen_connection(self):
		"""reopen"""
		print("MySQL > Explicitly opening connection ...")
		self.make_cursor()

	def close_connection(self):
		print("MySQL > Explicitly closing connection ...")
		self._cursor.close()
		self.mySQLConnection.close()

	@property
	def config(self):
		return self._config
		
	@config.setter
	def config(self, new_value):
		self._config = new_value
		self.filter_config()
		self.make_cursor()
	
	def filter_config(self):
		mustList = ['host','user','password']
		for key in self._config:
			if not key in mustList:
				ValueError(key,"should have in passed configuration")
				break

	def make_cursor(self):
		try:
			# Connection parameters and access credentials
			if 'database' in self._config:
				mySQLConnection = mysql.connect(
					host = self._config['host'],
					user = self._config['user'],
					passwd = self._config['password'],
					database = self._config['database']
				)
			else:
				mySQLConnection = mysql.connect(
					host = self._config['host'],
					user = self._config['user'],
					passwd = self._config['password']
				)
			self.mySQLConnection = mySQLConnection

		except mysql.errors.OperationalError:
			print("Error")
			# shaonutil.process.remove_aria_log('C:\\xampp\\mysql\\data')

		self._cursor = mySQLConnection.cursor()

	def is_mysql_user_exist(self,mysql_username):
		mySQLCursor = self._cursor
		mySqlListUsers = "select host, user from mysql.user;"
		mySQLCursor.execute(mySqlListUsers)
		userList = mySQLCursor.fetchall()
		foundUser = [user_ for host_,user_ in userList if user_ == mysql_username]
		if len(foundUser) == 0:
			return False
		else:
			return True

	def listMySQLUsers(self):
		"""list all mysql users"""
		mySQLCursor = self._cursor
		mySqlListUsers = "select host, user from mysql.user;"
		mySQLCursor.execute(mySqlListUsers)
		userList = mySQLCursor.fetchall()
		print("MySQL > List of users:")
		for user in userList:
		    host_,user_ = user
		    print("   ","host =",host_+",","user =",user_)


	def createMySQLUser(self, host, userName, password,
	               querynum=0, 
	               updatenum=0, 
	               connection_num=0):
		cursor = self._cursor
		try:
			print("MySQL > Creating user",userName)
			sqlCreateUser = "CREATE USER '%s'@'%s' IDENTIFIED BY '%s';"%(userName,host,password)
			cursor.execute(sqlCreateUser)
		except Exception as Ex:
			print("Error creating MySQL User: %s"%(Ex));

	def grantMySQLUserAllPrivileges(self, host, userName,
	               querynum=0, 
	               updatenum=0, 
	               connection_num=0):
		cursor = self._cursor
		try:
			print("MySQL > Granting all PRIVILEGES to user",userName)
			sqlGrantPrivilage = "GRANT ALL PRIVILEGES ON * . * TO '%s'@'%s';"%(userName,host)
			cursor.execute(sqlGrantPrivilage)
			cursor.execute("FLUSH PRIVILEGES;")

		except Exception as Ex:
			print("Error creating MySQL User: %s"%(Ex));

	def is_db_exist(self,dbname):
		cursor = self._cursor

		cursor.execute("SHOW DATABASES")
		databases = cursor.fetchall()
		databases = [x[0] for x in databases]
		if dbname not in databases:
			return False
		else:
			return True

	def create_db(self,dbname):
		cursor = self._cursor
		print("MySQL > Creating database "+dbname+" ...")
		cursor.execute("CREATE DATABASE "+dbname)

	def is_table_exist(self,tbname):
		cursor = self._cursor
		cursor.execute('SHOW TABLES')
		tables = cursor.fetchall()
		tables = [ x[0] for x in tables ]
		if tbname in tables:
			return True
		else:
			return False

	def create_table(self,tbname,column_info):
		cursor = self._cursor
		print("MySQL > Creating table "+tbname+" ...")
		cursor.execute("CREATE TABLE "+tbname+" (id INT(11) NOT NULL AUTO_INCREMENT PRIMARY KEY,"+''.join([' '+info+' '+column_info[info]+',' for info in column_info])[:-1]+")")

	def get_columns(self,tbname):
		cursor = self._cursor
		cursor.execute("SHOW COLUMNS FROM "+tbname)
		columns = cursor.fetchall()
		return columns
		# [('id', 'int(11)', 'NO', 'PRI', None, 'auto_increment'), ('book_sid', 'varchar(255)', 'YES', '', None, ''), ('book_name', 'varchar(255)', 'YES', '', None, ''), ('book_writer', 'varchar(255)', 'YES', '', None, ''), ('book_publisher', 'varchar(255)', 'YES', '', None, '')]

	def get_columns_names(self,tbname):
		columns = self.get_columns(tbname)
		return [column_name for column_name, *_ in columns]

	def get_unique_id_from_field(self,key_length,field_name):
		table = self._config['table']

		cursor = self._cursor

		sid = generateCryptographicallySecureRandomString(key_length)
		
		while True:
			query = "SELECT * FROM "+table+" WHERE `"+field_name+"` = '"+sid+"'"
			print(query)

			## getting records from the table
			cursor.execute(query)

			## fetching all records from the 'cursor' object
			records = cursor.fetchall()

			## Showing the data
			for record in records:
			    print(record)


			if(len(records)>1):
				print("matched with previously stored sid")
				sid = generateCryptographicallySecureRandomString(key_length)
			else:
				print("Got unique sid")
				break
				
		return sid

	def insert_data(self,value_tupple):
		cursor = self._cursor
		dbname = self._config['database']
		tbname = self._config['table']
		column_info = self.get_columns_names(tbname)[1:]
		
		# candidate_name, candidate_age, candidate_distance, candidate_living_place, candidate_university_or_instituition, candidate_image_webp_url, candidate_unique_image_name
		#cursor.execute("DESC "+tbname)
		
		query = "INSERT INTO "+tbname+ ' (' + ''.join([key+', ' for key in column_info])[:-2] + ') VALUES ('+ ''.join(['%s, ' for key in column_info])[:-2]  +')'
		
		## storing values in a variable
		values = [
		    value_tupple
		]

		## executing the query with values
		cursor.executemany(query, values)

		## to make final output we have to run the 'commit()' method of the database object
		self.mySQLConnection.commit()

		print(cursor.rowcount, "record inserted")

def create_configuration(option='cli'):
	"""Creating Configuration"""
	if option == 'cli':
		print('Getting your configurations to save it.\n')
		print('\nDatabase configurations -')
		dbhost = input('Give your db host : ')
		dbuser = input('Give your db user : ')
		dbpassword = input('Give your db password : ')
		dbname = input('Give your db name : ')
		dbtable = input('Give your db table : ')

		mysql_bin_folder = input('Give your path of mysql bin folder : ')

		f = open("private/config.ini", "w+")
		f.writelines(["; config file\n", "[db_authentication]\n", "host = "+dbhost+"\n", "user = "+dbuser+"\n", "password = "+dbpassword+"\n", "database = "+dbname+"\n", "table = "+dbtable+"\n","[MYSQL]\n","mysql_bin_folder = "+mysql_bin_folder+"\n","[DB_INITIALIZE]\n","host = localhost\n","usr = root\n","passwd = \n"])
		f.close()
	elif option == 'gui':
		window = Tk()
		window.title("Welcome to DB Config")
		window.geometry('400x400')
		window.configure(background = "grey");

		# Label fb_authentication
		FB_LABEL = Label(window ,text = "MYSQL Config").grid(row = 0,column = 0,columnspan=2)
		a = Label(window ,text = "MYSQL bin folder").grid(row = 1,column = 0)
		
		DB_LABEL = Label(window ,text = "Database Authentication").grid(row = 3,column = 0,columnspan=2)
		c = Label(window ,text = "Host").grid(row = 4,column = 0)
		d = Label(window ,text = "User").grid(row = 5,column = 0)
		d = Label(window ,text = "Password").grid(row = 6,column = 0)
		d = Label(window ,text = "Database").grid(row = 7,column = 0)
		d = Label(window ,text = "Table").grid(row = 8,column = 0)

		mysqlbinfolder_ = tk.StringVar(window)
		fbpassword_ = tk.StringVar(window)
		dbhost_ = tk.StringVar(window)
		dbuser_ = tk.StringVar(window)
		dbpassword_ = tk.StringVar(window)
		dbname_ = tk.StringVar(window)
		dbtable_ = tk.StringVar(window)

		Entry(window,textvariable=mysqlbinfolder_).grid(row = 1,column = 1)
		
		Entry(window,textvariable=dbhost_).grid(row = 4,column = 1)
		Entry(window,textvariable=dbuser_).grid(row = 5,column = 1)
		Entry(window,show="*",textvariable=dbpassword_).grid(row = 6,column = 1)
		Entry(window,textvariable=dbname_).grid(row = 7,column = 1)
		Entry(window,textvariable=dbtable_).grid(row = 8,column = 1)

		def clicked():
			mysql_bin_folder = mysqlbinfolder_.get()
			
			dbhost = dbhost_.get()
			dbuser = dbuser_.get()
			dbpassword = dbpassword_.get()
			dbname = dbname_.get()
			dbtable = dbtable_.get()

			f = open("private/config.ini", "w+")
			f.writelines(["; config file\n", "[db_authentication]\n", "host = "+dbhost+"\n", "user = "+dbuser+"\n", "password = "+dbpassword+"\n", "database = "+dbname+"\n", "table = "+dbtable+"\n","[MYSQL]\n","mysql_bin_folder = "+mysql_bin_folder+"\n","[DB_INITIALIZE]\n","host = localhost\n","usr = root\n","passwd = \n"])
			f.close()
			window.destroy()



		btn = ttk.Button(window ,text="Submit",command=clicked).grid(row=9,column=0)
		window.mainloop()


"""Process"""
import psutil
import wmi

import os
import subprocess

"""
To Resolve mysql problems:
go to mysql folder > aria_chk -r
delete araia_log.### files in mysql data folder
change the port=3306 to anything else in line 20,28 in my.ini in mysql data folder
if anything didn't recover then, go to mysqsl/backup, copy everything and go to mysql/data folder , delete everything and past here.
"""

def is_process_exist(process_name):
	pids = []
	processes = list_processes()
	for process_id_,process_name_ in processes:
		if process_name_ == process_name:
			pids.append(process_id_)

	if len(pids) == 0:
		return False
	else:
		return pids

def kill_duplicate_process(process_name,log):
	"""Kill a process if there is more than one instance is running."""
	if log: print("Killing duplicate processes")
	pids = is_process_exist(process_name)
	if len(pids) > 1:
		for pid in pids[1:]:
			killProcess_ByPid(pid)
	


def killProcess_ByAll(PROCNAME): 
    for proc in psutil.process_iter():
        # check whether the process name matches
        if proc.name() == PROCNAME:
            proc.kill()

def killProcess_ByPid(pid):
    p = psutil.Process(pid)
    p.terminate() 

def list_processes(sort='name',save_file=False,log=False):
    # sort - name/pid
    processes = []

    c = wmi.WMI ()
    
    #print(process.ProcessId, process.Name)
    
    if sort == 'pid':
        processes_pid = {}
        for process in c.Win32_Process():
            processes_pid[process.ProcessId] = process

        for pid in sorted(processes_pid.keys()):
            processes.append(  processes_pid[pid] )

    elif sort == 'name':
        processes_name = {}
        process_strings = []

        for process in c.Win32_Process():
            processes_name[process.Name] = process
            process_strings.append( process.Name )

        process_strings.sort()
        processes = [processes_name[process_name] for process_name in process_strings]

        
    processes_ = []
    if save_file: file_ = open('running_list.txt','w+')
    for process in processes:
        if log: print(process.ProcessId, process.Name)
        processes_.append((process.ProcessId, process.Name))
        if save_file: file_.write(f'{process.ProcessId} {process.Name}\n')

        #process.__dict__
        """

        instance of Win32_Process
        {
            Caption = "System Idle Process";
            CreationClassName = "Win32_Process";
            CreationDate = "20200206235942.060650+360";
            CSCreationClassName = "Win32_ComputerSystem";
            CSName = "DESKTOP-LC92I9N";
            Description = "System Idle Process";
            Handle = "0";
            HandleCount = 0;
            KernelModeTime = "2248239062500";
            Name = "System Idle Process";
            OSCreationClassName = "Win32_OperatingSystem";
            OSName = "Microsoft Windows 10 Pro|C:\\Windows|\\Device\\Harddisk0\\Partition4";
            OtherOperationCount = "0";
            OtherTransferCount = "0";
            PageFaults = 2;
            PageFileUsage = 0;
            ParentProcessId = 0;
            PeakPageFileUsage = 0;
            PeakVirtualSize = "65536";
            PeakWorkingSetSize = 4;
            Priority = 0;
            PrivatePageCount = "0";
            ProcessId = 0;
            QuotaNonPagedPoolUsage = 0;
            QuotaPagedPoolUsage = 0;
            QuotaPeakNonPagedPoolUsage = 0;
            QuotaPeakPagedPoolUsage = 0;
            ReadOperationCount = "0";
            ReadTransferCount = "0";
            SessionId = 0;
            ThreadCount = 4;
            UserModeTime = "0";
            VirtualSize = "65536";
            WindowsVersion = "10.0.14393";
            WorkingSetSize = "4096";
            WriteOperationCount = "0";
            WriteTransferCount = "0";
        };
        """
    if save_file: file_.close()

    return processes_

def start_mysql_server(mysql_bin_folder,mysql_config_file):
	"""Start mysql server"""
	DETACHED_PROCESS = 0x00000008	
	process = subprocess.Popen([os.path.join(mysql_bin_folder,"mysqld.exe"),"--defaults-file="+mysql_config_file,"--standalone"],creationflags=DETACHED_PROCESS)
	print("Starting mysql server at pid",process.pid)
	return process

def remove_aria_log(mysql_data_dir):
	"""Removing aria_log.### files to in mysql data dir to restart mysql"""
	aria_log_files = [file for file in os.listdir(mysql_data_dir) if 'aria_log.' in file]

	for aria_log in aria_log_files:
		aria_log = os.path.join(mysql_data_dir,aria_log)
		os.remove(aria_log)

def get_mysql_datadir(mysql_bin_folder,user,pass_=''):
	"""Get mysql data directory"""
	process = subprocess.Popen([os.path.join(mysql_bin_folder,"mysql"),"--user="+user,"--password="+pass_,"-e","select @@datadir;"],stdout=subprocess.PIPE)
	out, err = process.communicate()
	out = [line for line in out.decode('utf8').replace("\r\n","\n").split('\n') if line != ''][-1]
	datadir = out.replace('\\\\','\\')
	return datadir


#myNowMYSQL = MySQL(config._sections['DB_INITIALIZE'])
#{'host': 'localhost', 'user': 'root', 'password': ''}
#myNowMYSQL.config = config._sections['DB_AUTHENTICATION']



def computer_idle_mode():
    kill_running_app_list = ['kited.exe','sublime_text.exe','AoE2DE_s.exe','BattleServer.exe','conhost.exe','python.exe']



def obj_details_dump(obj):
	"""check dump"""
	#obj_details_dump(shaonutil)
	for attr in dir(obj):
	    print("obj.%s = %r" % (attr, getattr(obj, attr)))

#list_processes(log=True)
#killProcess_ByAll("chromedriver.exe")
#killProcess_ByAll("mysqld.exe")





"""Windows"""
import ctypes, sys
def is_winapp_admin():
    try:
        return ctypes.windll.shell32.IsUserAnAdmin()
    except:
        return False

def get_UAC_permission(func):
	if is_winapp_admin():
	    # Code of your program here
	    func()
	else:
	    # Re-run the program with admin rights
	    ctypes.windll.shell32.ShellExecuteW(None, "runas", sys.executable, __file__, None, 1)